"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const tslib_1 = require("tslib");
const shard_1 = require("./shard");
const crypto_1 = require("@dstack-js/crypto");
const pubsub_1 = require("../pubsub");
class Store {
    /**
     * stores shards linked to unique keys
     * @param ipfs IPFS instance
     * @param key unique key for store, used for shard replication
     * @param publicKey RSA Public Key, required for verifying data
     * @param privateKey RSA Private Key, required only if you need to add data
     */
    constructor(ipfs, key, publicKey, privateKey) {
        this.ipfs = ipfs;
        this.key = key;
        this.publicKey = publicKey;
        this.privateKey = privateKey;
        this.shardMap = {};
        this.crypto = new crypto_1.Crypto();
        this.crypto.setPublicKey(publicKey);
        if (privateKey)
            this.crypto.setPrivateKey(privateKey);
        this.replication = new pubsub_1.PubSub(`${this.key}/replication`, ipfs);
        this.replication.subscribe('new', ([key, shard]) => {
            const verified = this.crypto.verify(this.getSignatureData(key, shard), shard.extensions.signature);
            if (!verified) {
                console.warn('shard replication skipped for:', key, shard, 'reason: invalid signature');
                return;
            }
            if (this.shardMap[key] && shard.timestamp < this.shardMap[key].timestamp) {
                console.warn('shard replication skipped for:', key, shard, 'reason: shard is older than in shard map');
                return;
            }
            this.shardMap[key] = shard;
        });
    }
    getSignatureData(key, shard) {
        return `${key}:${shard.cid}`;
    }
    /**
     * add data to store
     * @param key unique key for data to be stored
     * @param data to be stored
     * @returns shard
     */
    add(key, data, replicate = true) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.shardMap[key] = yield (0, shard_1.addShard)(this.ipfs, data, { signature: '' });
            this.shardMap[key].extensions.signature = this.crypto.sign(this.getSignatureData(key, this.shardMap[key]));
            if (replicate)
                yield this.replication.publish('new', [key, this.shardMap[key]]);
            return this.shardMap[key];
        });
    }
    /**
     * get data from store
     * @param key unique key
     * @returns shard
     */
    get(key) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            // TODO: shard request on pubsub
            if (!this.shardMap[key])
                return null;
            let verified = true;
            if (this.publicKey && this.shardMap[key].extensions.signature) {
                verified = this.crypto.verify(this.getSignatureData(key, this.shardMap[key]), this.shardMap[key].extensions.signature);
            }
            if (!verified)
                return null;
            return this.shardMap[key];
        });
    }
    close() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.ipfs.pubsub.unsubscribe(this.key);
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map